@@ -1,35 +1,35 @@
 # import required packages
 import numpy as np
-from joblib import dump, load
 from sklearn.linear_model import Ridge
-from sklearn.metrics import mean_squared_error
-from sklearn.metrics import make_scorer
+from sklearn.metrics import mean_squared_error, make_scorer
 from sklearn.preprocessing import MinMaxScaler
-from sklearn.model_selection import TimeSeriesSplit
-from sklearn.model_selection import GridSearchCV
+from sklearn.model_selection import TimeSeriesSplit, GridSearchCV
 
 from matplotlib import pyplot as plt
 plt.rcParams["font.family"] = "Times New Roman"
 plt.rcParams["font.size"] = 10
 
-from pyrcn.base import InputToNode, NodeToNode
 from pyrcn.echo_state_network import ESNRegressor
 from pyrcn.extreme_learning_machine import ELMRegressor
 from pyrcn.linear_model import IncrementalRegression
 from pyrcn.model_selection import SequentialSearchCV
+from pyrcn.datasets import mackey_glass
 
 # Load the dataset
-X = np.loadtxt("./examples/dataset/MackeyGlass_t17.txt")
-X = MinMaxScaler(feature_range=(-1, 1)).fit_transform(X=X.reshape(-1,1))
+X, y = mackey_glass(n_timesteps=20000)
+scaler = MinMaxScaler(feature_range=(-1, 1)).fit(X=X)
+X = scaler.transform(X=X)
+y = scaler.transform(y)
 
 # Define Train/Test lengths
 trainLen = 1900 # number of time steps during which we train the network
 testLen = 2000 # number of time steps during which we test/run the network
 
-X_train = X[0:trainLen].reshape(-1, 1)
-y_train = X[1:trainLen+1].reshape(-1, 1)
-X_test = X[trainLen:trainLen+testLen].reshape(-1, 1)
-y_test = X[trainLen+1:trainLen+testLen+1].reshape(-1, 1)
+
+X_train = X[:trainLen]
+y_train = y[:trainLen]
+X_test = X[trainLen:trainLen+testLen]
+y_test = y[trainLen:trainLen+testLen]
 
 fig = plt.figure()
 plt.plot(X_train, label="Training input")
@@ -43,13 +43,10 @@ fig.set_size_inches(4, 2.5)
 plt.savefig('input_data.pdf', bbox_inches='tight', pad_inches=0)
 
 # initialize an ESNRegressor
-esn = ESNRegressor(input_to_node=InputToNode(),
-                   node_to_node=NodeToNode(),
-                   regressor=Ridge())  # IncrementalRegression()
+esn = ESNRegressor()  # IncrementalRegression()
 
 # initialize an ELMRegressor
-elm = ELMRegressor(input_to_node=InputToNode(),
-                   regressor=IncrementalRegression())  # Ridge()
+elm = ELMRegressor(regressor=Ridge())  # Ridge()
 
 # train a model
 esn.fit(X=X_train, y=y_train)
@@ -66,29 +63,23 @@ unit_impulse = np.zeros(shape=(100, 1), dtype=int)
 unit_impulse[5] = 1
 
 # Echo State Network sequential hyperparameter tuning
-initially_fixed_esn_params = {'input_to_node__hidden_layer_size': 100,
-                              'input_to_node__activation': 'identity',
-                              'input_to_node__k_in': 10,
-                              'input_to_node__random_state': 42,
-                              'input_to_node__bias_scaling': 0.0,
-                              'input_to_node__k_in': 1,
-                              'input_to_node__random_state': 42,
-                              'node_to_node__hidden_layer_size': 100,
-                              'node_to_node__activation': 'tanh',
-                              'node_to_node__leakage': 1.0,
-                              'node_to_node__bias_scaling': 0.0,
-                              'node_to_node__bi_directional': False,
-                              'node_to_node__k_rec': 10,
-                              'node_to_node__wash_out': 0,
-                              'node_to_node__continuation': False,
-                              'node_to_node__random_state': 42,
-                              'regressor__alpha': 1e-5,
-                              'random_state': 42 }
-}
-step1_esn_params = {'input_to_node__input_scaling': np.linspace(0.1, 5.0, 50),
-                    'node_to_node__spectral_radius': np.linspace(0.0, 1.5, 16)}
-step2_esn_params = {'node_to_node__leakage': np.linspace(0.1, 1.0, 10)}
-step3_esn_params = {'node_to_node__bias_scaling': np.linspace(0.0, 1.5, 16)}
+initially_fixed_esn_params = {'hidden_layer_size': 100,
+                              'input_activation': 'identity',
+                              'bias_scaling': 0.0,
+                              'random_state': 42,
+                              'k_in': 1,
+                              'reservoir_activation': 'tanh',
+                              'leakage': 1.0,
+                              'bi_directional': False,
+                              'k_rec': 10,
+                              'wash_out': 0,
+                              'continuation': False,
+                              'alpha': 1e-5 }
+
+step1_esn_params = {'input_scaling': np.linspace(0.1, 5.0, 50),
+                    'spectral_radius': np.linspace(0.0, 1.5, 16)}
+step2_esn_params = {'leakage': np.linspace(0.1, 1.0, 10)}
+step3_esn_params = {'bias_scaling': np.linspace(0.0, 1.5, 16)}
 
 scorer = make_scorer(score_func=mean_squared_error, greater_is_better=False)
 
@@ -96,7 +87,7 @@ kwargs = {'verbose': 5,
           'scoring': scorer,
           'n_jobs': -1}
 
-esn = ESNRegressor(input_to_node=InputToNode(), node_to_node=NodeToNode(), regressor=Ridge()).set_params(**initially_fixed_esn_params)
+esn = ESNRegressor(regressor=Ridge(), **initially_fixed_esn_params)
 
 
 ts_split = TimeSeriesSplit()
@@ -106,7 +97,6 @@ searches = [('step1', GridSearchCV, step1_esn_params, kwargs),
 
 
 sequential_search_esn = SequentialSearchCV(esn, searches=searches).fit(X_train, y_train)
-dump(sequential_search_esn, "sequential_search_esn.joblib")
 
 esn_step1 = sequential_search_esn.all_best_estimator_["step1"]
 esn_step2 = sequential_search_esn.all_best_estimator_["step2"]
@@ -161,20 +151,15 @@ plt.grid()
 fig.set_size_inches(2, 1.25)
 plt.savefig('BiasScalingESN.pdf', bbox_inches = 'tight', pad_inches = 0)
 
-
 # Extreme Learning Machine sequential hyperparameter tuning
-initially_fixed_elm_params = {'input_to_node__hidden_layer_size': 100,
-                              'input_to_node__activation': 'tanh',
-                              'input_to_node__k_in': 10,
-                              'input_to_node__random_state': 42,
-                              'input_to_node__bias_scaling': 0.0,
-                              'input_to_node__k_in': 1,
-                              'input_to_node__random_state': 42,
-                              'regressor__alpha': 1e-5,
-                              'random_state': 42 
-}
-step1_elm_params = {'input_to_node__input_scaling': np.linspace(0.1, 5.0, 50)}
-step2_elm_params = {'input_to_node__bias_scaling': np.linspace(0.0, 1.5, 16)}
+initially_fixed_elm_params = {'hidden_layer_size': 100,
+                              'activation': 'tanh',
+                              'k_in': 1,
+                              'alpha': 1e-5,
+                              'random_state': 42 }
+
+step1_elm_params = {'input_scaling': np.linspace(0.1, 5.0, 50)}
+step2_elm_params = {'bias_scaling': np.linspace(0.0, 1.5, 16)}
 
 scorer = make_scorer(score_func=mean_squared_error, greater_is_better=False)
 
@@ -182,23 +167,20 @@ kwargs = {'verbose': 10,
           'scoring': scorer,
           'n_jobs': -1}
 
-elm = ELMRegressor(input_to_node=InputToNode(), regressor=Ridge()).set_params(**initially_fixed_elm_params)
-
+elm = ELMRegressor(regressor=Ridge(), **initially_fixed_elm_params)
 
 ts_split = TimeSeriesSplit()
 searches = [('step1', GridSearchCV, step1_elm_params, kwargs),
             ('step2', GridSearchCV, step2_elm_params, kwargs)]
 
-
 sequential_search_elm = SequentialSearchCV(elm, searches=searches).fit(X_train, y_train)
-dump(sequential_search_elm, "sequential_search_elm.joblib")
 
 elm_step1 = sequential_search_elm.all_best_estimator_["step1"]
 elm_step2 = sequential_search_elm.all_best_estimator_["step2"]
 
 elm_step1.predict(X=unit_impulse)
 fig = plt.figure()
-im = plt.imshow(np.abs(elm_step1.hidden_layer_state.T),vmin=0, vmax=1.0)
+im = plt.imshow(np.abs(elm_step1.hidden_layer_state.T),vmin=0, vmax=0.3)
 plt.xlim([0, 100])
 plt.ylim([0, elm_step1.hidden_layer_state.shape[1] - 1])
 plt.xlabel('n')
@@ -210,7 +192,7 @@ plt.savefig('InputScaling.pdf', bbox_inches = 'tight', pad_inches = 0)
 
 elm_step2.predict(X=unit_impulse)
 fig = plt.figure()
-im = plt.imshow(np.abs(elm_step2.hidden_layer_state.T),vmin=0, vmax=1.0)
+im = plt.imshow(np.abs(elm_step2.hidden_layer_state.T),vmin=0, vmax=0.3)
 plt.xlim([0, 100])
 plt.ylim([0, elm_step2.hidden_layer_state.shape[1] - 1])
 plt.xlabel('n')
@@ -234,18 +216,13 @@ test_err_elm = mean_squared_error(y_true=y_test, y_pred=y_test_pred_elm)
 print("Test MSE ESN:\t{0}".format(test_err_esn))
 print("Test MSE ELM:\t{0}".format(test_err_elm))
 
-
-# Prediction of the training set.
-
-# In[ ]:
-
-
+# Prediction of the test set.
 fig = plt.figure()
 plt.plot(y_test_pred_esn, label="ESN prediction", linewidth=1)
 plt.plot(y_test_pred_elm, label="ELM prediction", linewidth=1)
 plt.plot(y_test, label="Test target", linewidth=.5, color="black")
 plt.xlabel("n")
-plt.xlim([0, 200])
+plt.xlim([0, 100])
 plt.ylabel("u[n]")
 plt.grid()
 plt.legend()
